@@ -34,6 +34,7 @@ char debug_id[] = "DEBUG - Copyright (C) 2000 Emile van Bergen.";
 #include <metatype.h> 	/* For meta_atoprt() and meta_ordtoa() */
 
 #include <debug.h>
+#include <platform.h>
 
 
 /*
@@ -179,7 +180,7 @@ void msg(int ourfac, int level, char *fmt, ...)
 	/* Prepare arglist and create formatted line */
 	va_start(ap, fmt);
 #ifdef HAVE_VSNPRINTF
-	vsnprintf(logline + n, 4000, fmt, ap);
+	vsnprintf(logline + n, sizeof(logline) - 16 - n, fmt, ap);
 #else
 	vsprintf(logline + n, fmt, ap);		/* Cross your fingers */
 #endif
