@@ -42,7 +42,7 @@ char textfile_id[] = "TEXTFILE - Copyright (C) 2000 Emile van Bergen.";
  */
 
 
-#include <malloc.h>	/* For malloc() / free() */
+#include <stdlib.h>	/* For malloc() / free() */
 #include <string.h>	/* For strcpy() / strcat() */
 #include <unistd.h>	/* For open() */
 #include <fcntl.h>	/* For O_RDONLY */
@@ -90,6 +90,9 @@ void  text_del(TEXT *t)
 
 		/* Free the basepath */
 		if (t->basepath) free(t->basepath);
+
+		/* Free the text object itself */
+		free(t);
 	}
 }
 
