@@ -21,6 +21,8 @@
  * 		      actually decimal, and the stack item has an underlying
  * 		      dictionary item, we try to look up the value as a named
  * 		      constant associated with that string attribute.
+ * 2002/04/23 - EvB - Sanitized conversion operators
+ * 		    - Fixed small ! operator behaviour bug on strings
  */
 
 char langvm[] = "LANGVM - Copyright (C) 2001 Emile van Bergen.";
@@ -97,6 +99,34 @@ void vm_del(VM *vm)
 }
 
 
+/* Utility functions for complex operators in vm_run */
+
+
+/* Convert date as in yyyy[mm[dd[hh[mm[ss]]]]] to seconds since Epoch */
+
+META_ORD op_datstr2ord(char *p, META_ORD l)
+{
+	ssize_t g;
+	struct tm tm;
+
+	memset(&tm, 0, sizeof(tm)); 
+	tm.tm_year = 1970; tm.tm_mon = 1; tm.tm_mday = 1;
+
+	(void)(
+	l && (tm.tm_year = meta_atoord(p, MIN(4,l), 0,0, &g, 10), p+=g, l-=g) &&
+	l && (tm.tm_mon  = meta_atoord(p, MIN(2,l), 0,0, &g, 10), p+=g, l-=g) &&
+	l && (tm.tm_mday = meta_atoord(p, MIN(2,l), 0,0, &g, 10), p+=g, l-=g) &&
+	l && (tm.tm_hour = meta_atoord(p, MIN(2,l), 0,0, &g, 10), p+=g, l-=g) &&
+	l && (tm.tm_min  = meta_atoord(p, MIN(2,l), 0,0, &g, 10), p+=g, l-=g) &&
+	l && (tm.tm_sec  = meta_atoord(p, MIN(2,l), 0,0, &g, 10)));
+
+	tm.tm_year -= 1900; tm.tm_mon--;
+	return mktime(&tm);
+}
+
+
+/* Utility macros for use inside vm_run */
+
 #define NEWSTR(str, len)						\
 	if (!((str) = (char *)malloc((len)))) {				\
 		ret = VM_NOMEM; break;					\
@@ -107,7 +137,6 @@ void vm_del(VM *vm)
 	(av)->p = (str); (av)->l = (len);				\
 	(av)->flags |= AV_FREE_P;
 
-
 #define L (stk[sp-2])
 #define R (stk[sp-1])
 
@@ -235,7 +264,8 @@ int vm_run(VM *vm)
 		   */
 
 		  case OP_MUL: L.l *=  R.l; break;
-		  case OP_DIV: L.l /=  R.l; break;
+		  case OP_DIV: if (R.l) { L.l /=  R.l; break; }
+			       ret = VM_INVALIDARG; break;
 		  case OP_CIDRMASK:
 		  	       L.l &= 0xffffffff << (32 - R.l); break;
 		  case OP_MOD: L.l %=  R.l; break;
@@ -366,7 +396,7 @@ int vm_run(VM *vm)
 
 		  case OP_NO:
 		  	meta_freeavdata(&R);
-			R.l = ! R.l;
+			R.l = ! R.l; R.p = 0; R.i = 0;
 			break;
 		  case OP_JMPZ:
 		  	if (! R.l) i += i->imm.d.disp;
@@ -379,7 +409,69 @@ int vm_run(VM *vm)
 		   * Conversion
 		   */
 
-		  case OP_ORD2RAW:
+		  case OP_OCTSTR2ORD:			/* oct digits -> ord */
+		  	if (!R.l) break;
+			n = meta_atoord(R.p, R.l, 0, 0, 0, 8);
+			meta_freeavdata(&R); R.p = 0; R.l = n; R.i = 0;
+			break;
+		  case OP_DECSTR2ORD:			/* dec digits -> ord */
+		  	if (!R.l) break;
+			n = meta_atoord(R.p, R.l, 0, 0, 0, 10);
+			meta_freeavdata(&R); R.p = 0; R.l = n; R.i = 0;
+			break;
+		  case OP_HEXSTR2ORD:			/* hex digits -> ord */
+		  	if (!R.l) break;
+			n = meta_atoord(R.p, R.l, 0, 0, 0, 16);
+			meta_freeavdata(&R); R.p = 0; R.l = n; R.i = 0;
+			break;
+		  case OP_RAWSTR2ORD:		 /* big-endian binary -> ord */
+		  	if (!R.l) break;
+			n = getord(R.p, R.l);
+			meta_freeavdata(&R); R.p = 0; R.l = n; R.i = 0;
+			break;
+		  case OP_IPASTR2ORD:	       /* dotted quad decimal -> ord */
+		  	if (!R.l) break;
+			n = meta_atoip(R.p, R.l, 0, 0, 0);
+			meta_freeavdata(&R); R.p = 0; R.l = n; R.i = 0;
+			break;
+		  case OP_DATSTR2ORD:		    /* YYYYMMDDhhmmss -> ord */
+		  	if (!R.l) break;
+			n = op_datstr2ord(R.p, R.l);
+		  	meta_freeavdata(&R); R.p = 0; R.l = n; R.i = 0;
+			break;
+		  case OP_INTSTR2ORD:	     /* digits or named const -> ord */
+		  	if (!R.l) break;
+			n = meta_atoord(R.p, R.l, 0, 0, &g, 0);
+			if (!g && R.i) {
+				META_VAL *val;
+				NEWSTR(s, R.l + 1); 
+				memcpy(s, R.p, R.l); s[R.l] = 0;
+				val = meta_getvalbyname(0, R.i, s);
+				if (val) n = val->nr;
+				free(s);
+			}
+			meta_freeavdata(&R); R.p = 0; R.l = n; R.i = 0;
+			break;
+
+		  case OP_ORD2OCTSTR:			/* ord -> oct digits */
+		  	NEWSTR(s, n = sizeof(META_ORD) * 3 + 2);
+			n = meta_ordtoa(s, n, 0, 10, R.l);
+			CHGSTR(&R, s, n);
+			R.i = 0;
+			break;
+		  case OP_ORD2DECSTR:			/* ord -> dec digits */
+		  	NEWSTR(s, n = sizeof(META_ORD) * 3 + 2);
+			n = meta_ordtoa(s, n, 0, 10, R.l);
+			CHGSTR(&R, s, n);
+			R.i = 0;
+			break;
+		  case OP_ORD2HEXSTR:			/* ord -> hex digits */
+		  	NEWSTR(s, n = sizeof(META_ORD) * 2 + 2);
+			n = meta_ordtoa(s, n, 0, 16, R.l);
+			CHGSTR(&R, s, n);
+			R.i = 0;
+			break;
+		  case OP_ORD2RAWSTR:		 /* ord -> big-endian binary */
 			if (R.i && R.i->val_size > 0) {
 				NEWSTR(s, R.i->val_size);
 				putord(s, R.i->val_size, R.l);
@@ -392,86 +484,30 @@ int vm_run(VM *vm)
 			}
 			R.i = 0;
 			break;
-		  case OP_ORD2DEC:
-		  	NEWSTR(s, n = sizeof(META_ORD) * 3 + 2);
-			n = meta_ordtoa(s, n, 0, 10, R.l);
-			CHGSTR(&R, s, n);
-			R.i = 0;
-			break;
-		  case OP_ORD2HEX:
-		  	NEWSTR(s, n = sizeof(META_ORD) << 1);
-			n = meta_ordtoa(s, n, 0, 16, R.l);
-			CHGSTR(&R, s, n);
-			R.i = 0;
-			break;
-		  case OP_ORD2DQ:
+		  case OP_ORD2IPASTR:	       /* ord -> dotted quad decimal */
 		  	NEWSTR(s, n = 20);
 			n = meta_iptoa(s, n, R.l);
 			CHGSTR(&R, s, n);
 			R.i = 0;
 			break;
-		  case OP_ORD2YMD:
-		  	NEWSTR(s, 10); memset(s, 0, 10);
-			t = R.l; strftime(s, 9, "%Y%m%d", gmtime(&t));
+		  case OP_ORD2DATSTR:		    /* ord -> YYYYMMDDhhmmss */
+		  	NEWSTR(s, 16); memset(s, 0, 15);
+			t = R.l; 
+			strftime(s, 15, "%Y%m%d%H%M%S", localtime(&t));
 			CHGSTR(&R, s, strlen(s));
 			R.i = 0;
 			break;
-		  case OP_ORD2HMS:
-		  	NEWSTR(s, 8); memset(s, 0, 8);
-			t = R.l; strftime(s, 7, "%H%M%S", gmtime(&t));
-			CHGSTR(&R, s, strlen(s));
-			R.i = 0;
-			break;
-		  case OP_ORD2YMDHMS:
-		  	NEWSTR(s, 16); memset(s, 0, 16);
-			t = R.l; strftime(s, 15, "%Y%m%d%H%M%S", gmtime(&t));
-			CHGSTR(&R, s, strlen(s));
-			R.i = 0;
-			break;
-		  case OP_ORD2DFMT:
+		  case OP_ORD2DFMSTR:	   /* ord -> strftime formatted date */
 		  	if (!R.l) { L.l = 0; break; }
 		  	NEWSTR(s, 128); memset(s, 0, 128);
 			NEWSTR(c, R.l + 1); memcpy(c, R.p, R.l); c[R.l] = 0;
 			t = L.l; 
-			strftime(s, 127, c, gmtime(&t));
+			strftime(s, 127, c, localtime(&t));
 			free(c);
 			CHGSTR(&L, s, strlen(s));
 			L.i = 0;
 			break;
 
-		  case OP_DEC2ORD:
-		  	if (!R.l) break;
-			n = meta_atoord(R.p, R.l, 0, 0, &g, 10);
-			if (!g && R.i) {
-				META_VAL *val;
-				NEWSTR(s, R.l + 1); 
-				memcpy(s, R.p, R.l); s[R.l] = 0;
-				val = meta_getvalbyname(0, R.i, s);
-				if (val) n = val->nr;
-				free(s);
-			}
-			meta_freeavdata(&R); R.p = 0; R.l = n; R.i = 0;
-			break;
-		  case OP_HEX2ORD:
-		  	if (!R.l) break;
-			n = meta_atoord(R.p, R.l, 0, 0, 0, 16);
-			meta_freeavdata(&R); R.p = 0; R.l = n; R.i = 0;
-			break;
-		  case OP_DQ2ORD:
-		  	if (!R.l) break;
-			n = meta_atoip(R.p, R.l, 0, 0, 0);
-			meta_freeavdata(&R); R.p = 0; R.l = n; R.i = 0;
-			break;
-		  case OP_YMD2ORD:
-		  	/* FIXME */
-		  	meta_freeavdata(&R); R.l = 0; R.p = 0; break;
-		  case OP_HMS2ORD:
-		  	/* FIXME */
-		  	meta_freeavdata(&R); R.l = 0; R.p = 0; break;
-		  case OP_YMDHMS2ORD:
-		  	/* FIXME */
-		  	meta_freeavdata(&R); R.l = 0; R.p = 0; break;
-
 		  /*
 		   * Assignment etc.
 		   */
