@@ -88,8 +88,8 @@ JOB *job_new(SOCK *s, time_t t)
 	   the data in the packet buffer, so don't free that packet buffer. */
 	reqh = meta_decode(s->c->m, s->c->ds_rad_pkt, 0, j->pkt, j->pktlen, 
 			   &reqt);
-	if (!reqh) { msg(F_RECV, L_NOTICE, "job_new: Could not decode packet"
-					   "!\n"); job_del(j); return 0; }
+	if (!reqh) { msg(F_RECV, L_ERR, "job_new: ERROR: Could not decode "
+					"packet!\n"); job_del(j); return 0; }
 
 	/* Add extra information about this request */
 	av = meta_addnewav(&reqh, &reqt, 0, 1, s->c->di_timestamp, 0, t, 0);
