@@ -15,6 +15,7 @@
  * History:
  * 2002/10/18 - EvB - Started, based on the old radclient test tool, Brian
  *                    Candler's enhancements and my RADIUS client for pppd.
+ * 2003/04/01 - EvB - Fixed typo in reply matching code
  */
 
 
@@ -54,7 +55,7 @@
 #define DEF_AUTHPORT		1812
 #define DEF_ACCTPORT		1813
 #define DEF_RETRYCOUNT		3
-#define DEF_TIMEOUT		2
+#define DEF_TIMEOUT		5
 
 
 /* exit codes; bit 6 == 0 means exit code == RAD-Code, 1 means error: */
@@ -392,7 +393,7 @@ void sock_handle_read(struct radcfg *c, time_t t)
 		rad_id);
 	    return;
 	}
-	n = rad_id & MAX_OUTREQBITS;
+	n = rad_id & MAX_OUTREQMASK;
 	r = &c->req[n];
 	if (r->pktlen <= 0) {
 	    msg(F_RECV, L_ERR, "Dropping answer: slot %d for id %d is empty!\n",
@@ -947,7 +948,7 @@ void usage(char *msg)
 		"  -l	use long attribute names in output\n"
 		"  -p	send given User-Password or CHAP-Password as-is\n"
 		"  -o	output response immediately, don't retry till valid\n"
-		"  -f	flood, send retrycount packets as fast as possible\n"
+/*		"  -f	flood, send retrycount packets as fast as possible\n" */
 		"  -q	quiet, no output on stderr\n"
 		"  -v	verbose\n"
 		"  -h	this help\n");
@@ -995,7 +996,7 @@ void parseoptions(int argc, char **argv, struct radcfg *c)
 		  case 'l': c->ioflags &= ~AVMSG_SHORTATTR; break;
 		  case 'p': c->noencpass = 1; break;
 		  case 'o': c->outimm = 1; break;
-		  case 'f': c->flood = 1; break;
+/*		  case 'f': c->flood = 1; break;		*/
 		  case 'q': for(fac = 0; fac < F_CNT; fac++) 
 				    msg_setthresh(fac, L_ERR);
 			    c->quiet = 1; 
